;; -*- mode: emacs-lisp; -*-

(require 'notmuch)

(add-hook 'message-send-hook 'anarcat/message-set-encryption)

(defun anarcat/message-set-encryption (&optional force)
  "Automatically set the encryption depending on the recipients.

If ran interactively, will overwrite settings, otherwise it will
not touch existing settings, so that this can safely be ran in
`message-send-hook'."
  (interactive "p")
  (message-goto-body)
  ;; clear existing settings
  (if (and force (re-search-forward "<#secure [^>]*>\n" nil t))
      (replace-match "" nil nil))
  ;; If we can encrypt, do so, else just sign.
  (if (or force (not (re-search-forward "<#secure [^>]*>\n" nil t)))
      (if (anarcat/message-guess-encryption)
          (insert "<#secure method=pgpmime mode=signencrypt>\n")
        (insert "<#secure method=pgpmime mode=sign>\n"))
    )
)

;; http://notmuchmail.org/pipermail/notmuch/2012/007340.html
(defun anarcat/message-guess-encryption ()
  "Return `t' if we have gpg public keys for all recipients of this message."
  (require 'epa-mail)
  (if (and (not (message-news-p))  ; No encryption for news, 
           (or (message-fetch-field "to") (message-fetch-field "cc"))) ; we have to have one address at least
      (catch :exit
        (mapc
         (lambda (addr)
           ;; check for a usable key - taken from epa-mail.el
           (if (not (epa-mail--find-usable-key
                     (epg-list-keys
                      (epg-make-context epa-protocol)
                      (downcase (mail-strip-quoted-names addr)))
                     'encrypt))
               (throw :exit nil)))
         (message-tokenize-header (concat
                                   (message-fetch-field "to")
                                   ","
                                   (message-fetch-field "cc"))))
        t)
    nil))
